"use strict";
/**
 * Copyright (C) 2018-present, Rimeto, LLC.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Proxies access to the given record object in a nullsafe manner.
 * To dereference a given value, invoke it as a function with optional
 * default value.
 *
 * For example, given:
 *   const x = {
 *     a: 'hello',
 *     b: {
 *       d: 'world',
 *     }
 *   }
 *
 * Then:
 *   x.a() === 'hello' &&
 *   x.b.d() === 'world' &&
 *   x.b.z() === undefined &&
 *   x.b.z('default value') === 'default value' &&
 *   z.y.z.a.b.c.d.e.f.g.h.i.j.k() === undefined
 */
function RecordProxy(data) {
    return new Proxy(((defaultValue) => (data !== undefined ? data : defaultValue)), {
        get: (target, key) => {
            const obj = target();
            if ('object' !== typeof obj) {
                return RecordProxy();
            }
            // Propagate proxy to Array elements if we're invoking an array method
            if (obj instanceof Array && Array.prototype[key] instanceof Function) {
                const proxied = obj.map((e) => RecordProxy(e));
                return proxied[key].bind(proxied);
            }
            return RecordProxy(obj[key]);
        },
    });
}
exports.RecordProxy = RecordProxy;
