"use strict";
/**
 * Copyright (C) 2018-present, Rimeto, LLC.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
describe('tform', () => {
    test('basic transforming', () => {
        const record = {
            job: 'Engineer ',
            name: 'John Doe',
            hobbies: 'Biking, Skating,,',
            address: {
                home: {
                    city: 'Cupertino',
                    zip: null,
                },
            },
        };
        const rules = {
            job: (X) => X.job(''),
            name: {
                // test deep rules
                first: (X) => X.name('abdc').split(' ')[0],
                last: (X) => X.name('').split(' ')[1],
            },
            age: (X) => X.age(-1),
            hobbies: (X) => index_1.splitList(',', X.hobbies('')),
            address: {
                home: (X) => X.address.home.city('').toLowerCase(),
                work: (X) => X.address.work.city('Unknown').toLowerCase(),
            },
        };
        const expected = {
            job: 'Engineer',
            name: {
                first: 'John',
                last: 'Doe',
            },
            age: -1,
            hobbies: ['Biking', 'Skating'],
            address: {
                home: 'cupertino',
                work: 'unknown',
            },
        };
        const tform = new index_1.Tform(rules);
        const output = tform.transform(record);
        expect(output).toEqual(expected);
    });
    test('basic error handling', () => {
        const rules = {
            error: () => {
                throw Error('oh noes!');
            },
            missing: (X) => X.foo(),
        };
        const record1 = { foo: 1 };
        const record2 = {};
        const tform = new index_1.Tform(rules);
        expect(tform.transform(record1)).toEqual({ missing: 1 });
        expect(tform.transform(record2)).toEqual({ missing: undefined });
        expect(tform.getErrors()).toEqual([
            {
                error: Error('oh noes!'),
                field: 'error',
                recordId: undefined,
                recordNo: 1,
                recordRaw: { foo: 1 },
            },
            {
                error: Error('oh noes!'),
                field: 'error',
                recordId: undefined,
                recordNo: 2,
                recordRaw: {},
            },
            {
                error: Error("Property 'missing' of result is undefined"),
                field: 'missing',
                recordId: undefined,
                recordNo: 2,
                recordRaw: {},
            },
        ]);
    });
    test('error reporting of record id', () => {
        const rules = {
            missing: (X) => X.foo(),
        };
        const record1 = { pk: 1 };
        const record2 = {};
        const tform = new index_1.Tform(rules, 'pk');
        expect(tform.transform(record1)).toEqual({});
        expect(tform.transform(record2)).toEqual({});
        expect(tform.getErrors()).toEqual([
            {
                error: Error("Property 'missing' of result is undefined"),
                field: 'missing',
                recordId: '1',
                recordNo: 1,
                recordRaw: { pk: 1 },
            },
            {
                error: TypeError("Missing ID key 'pk'"),
                recordNo: 2,
                recordRaw: {},
            },
            {
                error: Error("Property 'missing' of result is undefined"),
                field: 'missing',
                recordId: undefined,
                recordNo: 2,
                recordRaw: {},
            },
        ]);
    });
});
